#ifndef __PDP_H_INCLUDED__
#define __PDP_H_INCLUDED__

#include <sys/fcntl.h>
#include <parsec/pdp_common.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
Инициализация интерфейса подсистемы безопасности PDP
@return 0 в случа успеха
*/
int pdp_init(void);

/**
Освобождение ресурсов интерфейса подсистемы безопасности PDP
@return 0 в случа успеха
*/
int pdp_release(void);

/**
Добавление роли в набор ролей метки безопасности
@param id идентификатор роли (номер inode)
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_add_role(ino_t id, PDPL_T* l);

/**
Удаление роли из набора ролей метки безопасности
@param id идентификатор роли (номер inode)
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_rm_role(ino_t id, PDPL_T* l);

/**
Очистка набора ролей метки безопасности
@param l указатель на метку безопасности
*/
void pdp_clear_roles(PDPL_T* l);

/**
Добавление административной роли в набор административных ролей метки безопасности
@param id идентификатор роли (номер inode)
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_add_arole(ino_t id, PDPL_T* l);

/**
Удаление административной роли из набора административных ролей метки безопасности
@param id идентификатор роли (номер inode)
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_rm_arole(ino_t id, PDPL_T* l);

/**
Очистка набора административных ролей метки безопасности
@param l указатель на метку безопасности
*/
void pdp_clear_aroles(PDPL_T* l);

///**
//Получение метки безопасности из бинарного представления

//@param buf указатель на буфер, который содержти метку безопасности в бинарном виде
//@param size размер буфера
//@return указатель на метку безопасности или NULL в случае ошибки
//*/
//PDP_O* pdp_from_raw(const void *buf, size_t size PDP_KRNL(gfp_t f));

/**
Получение бинарного представления метки безопасности

@param l указатель на метку безопасности
@param size указатель на переменную, куда возвращается размер выделенного буфера (НЕ NULL)
@return указатель на буфер, выделенный функцией malloc(), в случае ошибки NULL
*/
void* pdp2raw(const PDPL_T* l, size_t* size); /* буфер будет выделен функцией */

#define PDPL_FMT_TXT            0x00
#define PDPL_FMT_NUM            0x01
//#define PDPL_LEV              0x02
//#define PDPL_ILEV             0x04
//#define PDPL_CAT              0x08
//#define PDPL_TYPE             0x10
#define PDPL_FMT_NO_FORCE       0x20
#define PDPL_FMT_NO_PRINT_ZEROS 0x40

#include <obstack.h>
#ifndef obstack_chunk_alloc
#define obstack_chunk_alloc malloc
#endif
#ifndef obstack_chunk_free
#define obstack_chunk_free free
#endif

/**
Получение из стека непрерывной строки, которая должна быть освобождена free()
@param os - стек
@return указатель на строку или NULL
*/
char* pdp_obstack_finish(struct obstack *os);

/* pdp*_obstack ------------------------------------------------------------ */

/**
Вывод в объектный стек текстового представдения уровня конф.
@param l уровень конф.
@param os объектный стек
@param flags флаги (на данный момент только PDPL_FMT_TXT,PDPL_FMT_NO_FORCE)
@return аналогично возвращаемому значению obstack_printf()
*/
int pdp_lev_obstack(PDP_LEV_T l, struct obstack *os, int flags);

/**
Вывод в объектный стек текстового представдения уровня целостности
@param il уровень целостности
@param os объектный стек
@param flags флаги (на данный момент только PDPL_FMT_TXT,PDPL_FMT_NO_FORCE)
@return аналогично возвращаемому значению obstack_printf()
*/
int pdp_ilev_obstack(PDP_ILEV_T il, struct obstack *os, int flags);

/**
Вывод в объектный стек текстового представдения категорий метки безопасности
@param с категории метки безопасности
@param os объектный стек
@param flags флаги (на данный момент только PDPL_FMT_TXT,PDPL_FMT_FORCE)
@return аналогично возвращаемому значению obstack_printf()
*/
int pdp_cat_obstack(PDP_CAT_T c, struct obstack *os, int flags);

/**
Вывод в объектный стек текстового представдения типа метки безопасности
@param t тип метки безопасности
@param os объектный стек
@param flags флаги (на данный момент только PDPL_FMT_TXT,PDPL_FMT_FORCE)
@return аналогично возвращаемому значению obstack_printf()
*/
int pdp_type_obstack(PDP_TYPE_T t, struct obstack* os, int flags);

int pdpml_obstack(const PDPML_T *ml, struct obstack* os, int flags);
int pdprl_obstack(const PDPRL_T *rl, struct obstack* os, int flags);
int pdpl_obstack(const PDPL_T *l, struct obstack* os, int flags);

/* pdp*l*_obstack_printf ---------------------------------------------------- */

/**
Вывод в объектный стек текстового представдения маднатной метки в произвольном формате
@param ml указатель на мандатную метку
@param os объектный стек
@param fmt формат
@return аналогично obstack_printf (количество байт в случае успеха, иначе -1)
*/
int pdpml_obstack_printf(const PDPML_T *ml, struct obstack* os, const char* fmt,...);

/**
Вывод в объектный стек текстового представдения ролевой метки в произвольном формате
@param rl указатель на ролевую метку
@param os объектный стек
@param fmt формат
@return аналогично obstack_printf (количество байт в случае успеха, иначе -1)
*/
int pdprl_obstack_printf(const PDPRL_T *rl, struct obstack* os, const char* fmt, ...);

/**
Формирование текстового представления метки безопасности на стеке в произвольном формате
@param l указатель на метку безопасности
@param o стек
@param fmt формат
@return аналогично obstack_printf (количество байт в случае успеха, иначе -1)
*/
int pdpl_obstack_printf(const PDPL_T *l, struct obstack* os, const char* fmt, ...);

/* pdp*l*_asprintf ---------------------------------------------------------- */

/**
Формирование текстового представления мандатной безопасности в виде строки в произвольном формате
@param ml указатель на мандатную метку
@param fmt формат
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdpml_asprintf(const PDPML_T *ml, const char* fmt, ...);

/**
Формирование текстового представления ролевой метки в виде строки в произвольном формате
@param rl указатель на ролевую метку
@param fmt формат
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdprl_asprintf(const PDPRL_T *Rl, const char* fmt, ...);

/**
Формирование текстового представления метки безопасности в виде строки в произвольном формате
@param l указатель на метку безопасности
@param fmt формат
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdpl_asprintf(const PDPL_T *l, const char* fmt,...);

/* pdp*_get_text ------------------------------------------------------------ */

/**
Получение текстового представления уровня конфиденциальности
@param l уровень конфиденциальности
@param f флаги (на данный момент только PDPL_FMT_FORCE)
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdp_lev_get_text(PDP_LEV_T l, int flags);

/**
Получение текстового представдения уровня целостности
@param il уровень целостности
@param f флаги (на данный момент только PDPL_FMT_FORCE)
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdp_ilev_get_text(PDP_ILEV_T il, int flags);

/**
Получение текстового представдения категорий
@param c категории
@param f флаги (на данный момент только PDPL_FMT_TXT, PDPL_FMT_NO_FORCE)
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdp_cat_get_text(PDP_CAT_T c, int flags);

/**
Получение текстового представдения типа мандатной метки
@param t тип мандатной метки
@param f флаги (на данный момент только PDPL_FMT_TXT, PDPL_FMT_NO_FORCE)
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdp_type_get_text(PDP_TYPE_T t, int flags);

/**
Получение текстового представления мандатной метки
@param ml указатель на мандатную метку
@param flags флаги
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdpml_get_text(const PDPML_T *ml, int flags); /* auto allocate  */

/**
Получение текстового представления ролевой метки
@param r указатель на ролевую метку
@param f флаги
@return указатель на строку, память под которую выделена посредством malloc(),
сожержащую список ролей и административных ролей в формате "role1,...,roleN:arole1,... ,aroleM",
в случае ошибки NULL
*/
char* pdprl_get_text(const PDPRL_T *rl, int f);

/**
Получение текстового представления метки безопасности
@param l указатель на метку безопасности
@param flags флаги
@return указатель на строку, память под которую выделена посредством malloc(),
в случае ошибки NULL
*/
char* pdpl_get_text(const PDPL_T* l, int f); /* auto allocate  */

/* pdp*_get_from_text ------------------------------------------------------- */

/**
Получение уровня конфиденциальности из текстового представления
@param txt текстовое представление уровня конфиденциальности или
 NULL (в этом случае ф-ция вернет 0 и *lev=0)
@param lev указатель на уровень конфиденциальности
@return 0 в случа успеха
*/
int pdp_lev_from_text(const char *txt, PDP_LEV_T *lev);

/**
Получение уровня целостности из текстового представления
@param txt текстовое представление уровня целостности или
 NULL (в этом случае ф-ция вернет 0 и *ilev=0)
@param ilev указатель на уровень целостности
@return 0 в случа успеха
*/
int pdp_ilev_from_text(const char *txt, PDP_ILEV_T *ilev);

/**
Получение категорий из текстового представления
@param txt текстовое представление категорий или
 NULL (в этом случае ф-ция вернет 0 и *c=0)
@param c указатель на категории
@return 0 в случа успеха
*/
int pdp_cat_from_text(const char *txt, PDP_CAT_T *c);

/**
Получение типа мандатной метки из текстового представления
@param txt текстовое представление типа мандатной метки или
 NULL (в этом случае ф-ция вернет 0 и *type=0)
@param type указатель на тип мандатной метки
@return 0 в случа успеха
*/
int pdp_type_from_text(const char *txt, PDP_TYPE_T *type);

/**
Получение мандатной метки процесса из текстового представления
@param text указатель на строку
@param roles
@return указатель на метку безопасности
*/
PDPML_T* pdpml_get_from_text(const char *text); /* from text */

/**
Получение ролей из текстового представления: role_seed1,role_seed2,role_seed3,...
@param _in txt текстовое представление ролей или NULL
@param _out l указатель на массив ролей
@param _out r_cnt возвращает число ролей в массивы
@return 0 в случае успеха, -1 в противном случае
*/
int _pdp_role_from_text(const char *txt, PDP_ROLE_T **r, uint32_t *r_cnt); /* auto allocate  */

int pdp_role_from_text(PDP_ROLE_T *r, const char *txt);
PDPRL_T* pdprl_get_from_raw(const void *raw, size_t size PDP_KRNL_ARG(gfp_t f));

/**
Получение ролевой метки из текстового представления
@param text указатель на строку,
сожержащую список ролей и административных ролей в формате "role_seed1,...,role_seedN!arole_seed1,... ,arole_seedM",
@return указатель на ролевую метку, в случае ошибки NULL
*/
PDPRL_T* pdprl_get_from_text(const char *text); /* auto allocate  */

/**
Получение метки безопасности из текстового представления
@param text указатель на строку
@return указатель на метку безопасности
*/
PDPL_T* pdpl_get_from_text(const char *text); /* from text */

/************************************************************************************/

/**
Установка метки безопасности на файл
@param path имя файла
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_set_path(const char *path, const PDPL_T* l);

/**
Установка метки безопасности файлового дескриптора
@param fd файловый дескриптор
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_set_fd(int fd, const  PDPL_T* l);

/**
Установка метки безопасности на символическую ссылку
@param path имя файла
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_set_lpath(const char *path, const PDPL_T* l);

/**
Получение метки безопасности файла
@param path имя файла
@return указатель на метку безопасности в случае успеха, иначе NULL
*/
PDPL_T* pdp_get_path(const char *path);

/**
Получение метки безопасности файлового дескриптора
@param fd файловый дескриптор
@return указатель на метку безопасности в случае успеха, иначе NULL
*/
PDPL_T*	pdp_get_fd(int fd);

/**
Получение метки безопасности символической ссылки
@param path имя файла
@return указатель на метку безопасности в случае успеха, иначе NULL
*/
PDPL_T* pdp_get_lpath(const char *path);

static inline int pdp_set_EQU_path(const char *path) { return pdp_set_path(path,PDPL_EQU); }
static inline int pdp_set_EQU_fd(int fd) { return pdp_set_fd(fd,PDPL_EQU); }

/**
Установка нулевой метки безопасности с флагом PDPT_EHOLE на файл
@param fd файловый дескриптор
@return 0 в случа успеха
*/
int pdp_set_ehole_fd(int fd);

/**
Установка нулевой метки безопасности с флагом PDPT_EHOLE на файл
@param path имя файла
@return 0 в случа успеха
*/
int pdp_set_ehole(const char *path);

/**
Получение максимальной метки безопасности в системе
@return аналогично pdp_get_path
*/
static inline PDPL_T* pdp_get_sys_max(void) { return pdp_get_path("/"); }

int pdp_set_sys_max_path(const char *path);
int pdp_set_sys_max_fd(int fd);

int pdp_set_sys_max_ccnr_path(const char *path);

/**
Установка метки безопасности процесса
@param pid идентификатор процесса
@param l указатель на метку безопасности
@return 0 в случа успеха
*/
int pdp_set_pid(pid_t pid, const PDPL_T* l);

/**
Получение метки безопасности процесса
@param pid идентификатор процесса
@return указатель на метку безопасности в случае успеха, иначе NULL
*/
PDPL_T* pdp_get_pid(pid_t pid);

/**
Установка метки безопасности текущего процесса
@param l указатель на метку безопасности
@return аналогично pdp_set_pid
*/
static inline int pdp_set_current(const PDPL_T* l) { return pdp_set_pid(0,l); }

/**
Получение метки безопасности текущего процесса
@return аналогично pdp_get_pid
*/
static inline PDPL_T* pdp_get_current(void) { return pdp_get_pid(0); }

/**
Получение максимального уровня целостности в системе
@return числовое значение уровня целостности
*/
PDP_ILEV_T pdp_get_max_ilev();

/**
Получение уровня целостности текущего процесса
@return числовое значение уровня целостности
*/
PDP_ILEV_T pdp_get_current_ilev();

/**
Получение списка числовых значений уровней целостности
@param count указатель на целое куда возвращается число уровней
@return ссылка на статический массив  целых чисел (уровней)
*/
int * pdp_get_ilevs(int * count);

/**
Возвращает 1 если тип корректен
@param l указатель на метку безопасности
@param type тип метки безопасности
@param mode значение stat st_mode объекта
@return ссылка на статический массив  целых чисел (уровней)
*/
int pdpl_supports_type(const PDPL_T *l, PDP_TYPE_T type, mode_t mode);

/**
Возвращает 1 если текущий процесс привилегированный и его уровень
целостности равен максимальному; иначе 0
*/
int pdp_is_super_root();

#ifdef __cplusplus
}
#endif

#endif /* __PDP_H_INCLUDED__ */

